/**
 * This jQuery plugin displays pagination links inside the selected elements.
 *
 * @author Gabriel Birke (birke *at* d-scribe *dot* de)
 * @version 1.2
 * @param {int} maxentries Number of entries to paginate
 * @param {Object} opts Several options (see README for documentation)
 * @return {Object} jQuery Object
 */
jQuery.fn.pagination = function(maxentries, opts){
	opts = jQuery.extend({
		items_per_page:10,
		per_pages:[10,20,50],
		num_display_entries:3,
		current_page:0,
		num_edge_entries:1,
		link_to:"javascript:;",
		prev_text:" ",
		next_text:" ",
		ellipse_text:"...",
		prev_show_always:true,
		next_show_always:true,
		callback:function(){return false;}
	},opts||{});

	if(opts.items_per_page==null||opts.items_per_page==undefined){
		if(typeof(opts.per_pages) == "number") {//如果传入的是数字，重新赋值每页展示多少
			opts.items_per_page=opts.per_pages;
		} else if(opts.per_pages.length>0){//如果传入的是数组，表示可以选择每页展示多少
					opts.items_per_page=opts.per_pages[0];
		}
	}
	return this.each(function() {
		/**
		 * 计算最大分页显示数目
		 */
		function numPages() {
			return Math.ceil(maxentries/opts.items_per_page);
		}
		/**
		 * 极端分页的起始和结束点，这取决于current_page 和 num_display_entries.
		 * @返回 {数组(Array)}
		 */
		function getInterval()  {
			var ne_half = Math.ceil(opts.num_display_entries/2);
			var np = numPages();
			var upper_limit = np-opts.num_display_entries;
			var start = current_page>ne_half?Math.max(Math.min(current_page-ne_half+(opts.num_display_entries%2==1?1:0), upper_limit), 0):0;
			var end = current_page>ne_half?Math.min(current_page+ne_half, np):Math.min(opts.num_display_entries+opts.num_edge_entries+1, np);
			return [start,end];
		}

		/**
		 * 分页链接事件处理函数
		 * @参数 {int} page_id 为新页码
		 */
		function pageSelected(page_id, evt){
			current_page = page_id;
			drawLinks();
			var continuePropagation = opts.callback({current_page:current_page,pageSize:opts.items_per_page}, panel);
			if (!continuePropagation) {
				if (evt.stopPropagation) {
					evt.stopPropagation();
				}
				else {
					evt.cancelBubble = true;
				}
			}
			return continuePropagation;
		}

		/**
		 * 此函数将分页链接插入到容器元素中
		 */
		function drawLinks() {
			panel.empty();
			var interval = getInterval();
			var np = numPages();
			// 这个辅助函数返回一个处理函数调用有着正确page_id的pageSelected。
			var getClickHandler = function(page_id) {
				return function(evt){ return pageSelected(page_id,evt); }
			}
			//辅助函数用来产生一个单链接(如果不是当前页则产生span标签)
			var appendItem = function(page_id, appendopts){
				page_id = page_id<0?0:(page_id<np?page_id:np-1); // 规范page id值
				appendopts = jQuery.extend({text:page_id+1, classes:""}, appendopts||{});
				if(page_id == current_page){
					var lnk = jQuery("<span class='current'>"+(appendopts.text)+"</span>");
				}else{
					var lnk = jQuery("<a class='pagination-num'>"+(appendopts.text)+"</a>")
						.bind("click", getClickHandler(page_id))
						.attr('href', opts.link_to.replace(/__id__/,page_id));
				}
				if(appendopts.classes){lnk.addClass(appendopts.classes);}
				panel.append(lnk);
			}
			// 产生"Previous"-链接
			if(opts.prev_text && (current_page > 0 || opts.prev_show_always)){
				appendItem(current_page-1,{text:opts.prev_text, classes:"prev"});
			}
			// 产生起始点
			if (interval[0] > 0 && opts.num_edge_entries > 0)
			{
				var end = Math.min(opts.num_edge_entries, interval[0]);
				for(var i=0; i<end; i++) {
					appendItem(i);
				}
				if(opts.num_edge_entries < interval[0] && opts.ellipse_text)
				{
					jQuery("<span>"+opts.ellipse_text+"</span>").appendTo(panel);
				}
			}
			// 产生内部的些链接
			for(var i=interval[0]; i<interval[1]; i++) {
				appendItem(i);
			}
			// 产生结束点
			if (interval[1] < np && opts.num_edge_entries > 0)
			{
				if(np-opts.num_edge_entries > interval[1]&& opts.ellipse_text)
				{
					jQuery("<span>"+opts.ellipse_text+"</span>").appendTo(panel);
				}
				var begin = Math.max(np-opts.num_edge_entries, interval[1]);
				for(var i=begin; i<np; i++) {
					appendItem(i);
				}

			}
			// 产生 "Next"-链接
			if(opts.next_text && (current_page < np-1 || opts.next_show_always)){
				appendItem(current_page+1,{text:opts.next_text, classes:"next"});
			}
            panel.append('<span class="pagination-total">共&nbsp;'+maxentries+'&nbsp;项</span>');
			if(opts.per_pages.length>0){
				var options=[],perSelect;
                panel.append('<span style="padding-right:2px;">每页</span>');
                for(var i=0; i<opts.per_pages.length; i++) {
                	if(opts.per_pages[i]==opts.items_per_page){
                        options.push('<option selected>'+opts.per_pages[i]+'</option>');
					}else{
                        options.push('<option>'+opts.per_pages[i]+'</option>');
					}
                }
                perSelect=jQuery('<select class="pagination-select">'+options.join('')+"</select>").bind('change',function(evt){
                    opts.items_per_page=parseInt(jQuery(this).find("option:selected").text());
                    return pageSelected(0,evt);
				});
                panel.append(perSelect);
                panel.append('<span style="padding-left:2px;">项</span>');
            }
            //如果有搜索第几页
			if(opts.go){
				panel.append('<label class="pagination-label" for="current_go">跳转到&nbsp;</label><input type="text" size="8" maxlength="8" onkeyup="this.value=this.value.replace(/\\D/g,\'\')" onafterpaste="this.value=this.value.replace(/\\D/g,\'\')"  class="pagination-text" id="current_go"/>&nbsp;页');
				var go = jQuery('<button class="pagination-go">'+(opts.go||'GO')+"</button>")
						.bind("click", function(evt){
							var current_go=jQuery('#current_go').val().replace(/\D/g,'');
							if(current_go.length!=0){
								current_go=Math.min(parseInt(current_go),np);
								if(current_go==0){current_go=1;}
								return pageSelected(current_go-1,evt);
							}else{
								jQuery('#current_go').val('');
							}
							return false;
						});
					panel.append(go);
			}

			/**
			 * 1.有数据，且大于可选每页条数的第一项，则显示
			 * 2.有数据，没有可选每页条数，页数大于1，则显示
			 * 3.不满足1或2，隐藏
			 */
			if(maxentries && (opts.per_pages.length>0? maxentries>opts.per_pages[0] : numPages()>1)) {
				panel.show();
			} else {
				panel.hide();
			}
		}
		//从选项中提取current_page
		var current_page = opts.current_page;
		//创建一个显示条数和每页显示条数值
		maxentries = (typeof(maxentries)=='undefined' || maxentries < 0)?0:maxentries;
		opts.items_per_page = (!opts.items_per_page || opts.items_per_page < 0)?1:opts.items_per_page;
		//存储DOM元素，以方便从所有的内部结构中获取
		var panel = jQuery(this);
		// 获得附加功能的元素
		this.selectPage = function(page_id){ pageSelected(page_id);}
		this.prevPage = function(){
			if (current_page > 0) {
				pageSelected(current_page - 1);
				return true;
			}
			else {
				return false;
			}
		}
		this.nextPage = function(){
			if(current_page < numPages()-1) {
				pageSelected(current_page+1);
				return true;
			}
			else {
				return false;
			}
		}
		// 所有初始化完成，绘制链接
		drawLinks();
        // 回调函数
        //opts.callback({current_page:current_page,pageSize:opts.items_per_page}, this);
	});
}
